@@ -301,6 +301,11 @@ def execute_generation(
         )
         stats.generated += 1
     shutil.rmtree(temp_dir, ignore_errors=True)
+    if hasattr(generator, "close"):
+        try:
+            generator.close()
+        except Exception:  # pragma: no cover - defensive cleanup
+            pass
     return stats
 
 
