@@ -814,6 +814,12 @@ def cmd_generate(args: argparse.Namespace) -> int:
     )
     manager.promote_variant(ctx, manifest, 0)
 
+    if hasattr(generator, "close"):
+        try:
+            generator.close()
+        except Exception:  # pragma: no cover - defensive cleanup
+            pass
+
     force_restart_gallery = args.restart_gallery or GALLERY_ALWAYS_RESTART
     _ensure_gallery_available(config, paths, GALLERY_HOST, GALLERY_PORT, force_restart_gallery)
 
