@@ -8,7 +8,7 @@ from concurrent.futures import ThreadPoolExecutor
 from ._compat import dataclass
 from io import BytesIO
 from pathlib import Path
-from typing import Dict, List, Optional, Protocol, Tuple
+from typing import Dict, List, Optional, Protocol, Set, Tuple
 
 import hashlib
 
@@ -122,7 +122,7 @@ class MockImageGenerator:
             variant_color = _shift_color(base_color, rng.randint(-40, 40))
             image = Image.new("RGB", (width, height), variant_color)
             draw = ImageDraw.Draw(image)
-            overlay_text = _truncate(prompt, 120) or "(empty prompt)"
+            overlay_text = _truncate_overlay(prompt, 120) or "(empty prompt)"
             info_text = f"{model or provider or 'mock'} #{index + 1}"
             _draw_text(draw, overlay_text, (width, height), headline=True)
             _draw_text(draw, info_text, (width, height), headline=False, y_offset=height - 80)
@@ -265,17 +265,25 @@ class OpenRouterGenerator:
 
             if not images:
                 debug_message = ""
+                reason = "empty_response"
+                failure_message: Optional[str] = None
+                text_snippets: List[str] = []
                 if last_response is not None:
                     try:
                         debug_message = json.dumps(last_response, indent=2)
                     except Exception:
                         debug_message = str(last_response)
+                    reason, failure_message, text_snippets = _diagnose_no_image_response(last_response)
+                message_text = failure_message or "OpenRouter did not return any image files."
+                context = {**request_context, "reason": reason}
+                if text_snippets:
+                    context["response_text"] = text_snippets
                 raise ProviderExecutionError(
                     provider=provider_name,
                     generator="openrouter",
-                    message="OpenRouter did not return any image files.",
+                    message=message_text,
                     body=_truncate(debug_message) if debug_message else None,
-                    context={**request_context, "reason": "empty_response"},
+                    context=context,
                 )
         except ProviderExecutionError as exc:
             exc.attach_context(request_context)
@@ -287,8 +295,6 @@ class OpenRouterGenerator:
                 message=str(exc),
                 context={**request_context, "reason": "unexpected_exception"},
             ) from exc
-        finally:
-            self.client.close()
 
         artifacts: List[GeneratedImageArtifacts] = []
         for index, entry in enumerate(images[:total]):
@@ -417,12 +423,7 @@ class OpenRouterGenerator:
             if provider_meta is not None:
                 metadata.append(provider_meta)
             message = choice.get("message") or {}
-            for image in message.get("images", []) or []:
-                data_url = image.get("image_url", {}).get("url")
-                media_type = image.get("image_url", {}).get("mime_type", "image/png")
-                if not data_url:
-                    continue
-                images.append({"data_url": data_url, "media_type": media_type})
+            images.extend(_extract_images_from_message(message))
         return images, warnings, metadata
 
     def _post_json(self, payload: Dict[str, object]) -> Dict[str, object]:
@@ -442,7 +443,7 @@ class OpenRouterGenerator:
 
         status = response.status_code
         if status >= 400:
-            body = _truncate(response.text or "")
+            body = _truncate(response.text or "", limit=800)
             context = {
                 "reason": "http_status",
                 "url": self.api_url,
@@ -470,10 +471,254 @@ class OpenRouterGenerator:
                 generator="openrouter",
                 message="OpenRouter response was not valid JSON.",
                 status=status,
-                body=_truncate(response.text or ""),
+                body=_truncate(response.text or "", limit=800),
                 context={"reason": "invalid_json", "url": self.api_url},
             ) from exc
 
+    def close(self) -> None:
+        try:
+            self.client.close()
+        except Exception:  # pragma: no cover - defensive cleanup
+            pass
+
+
+def _ensure_data_url(payload: str, media_type: Optional[str]) -> str:
+    media = media_type or "image/png"
+    cleaned = payload.strip()
+    if cleaned.startswith("data:"):
+        return cleaned
+    return f"data:{media};base64,{cleaned}"
+
+
+def _extract_images_from_message(message: Dict[str, object]) -> List[Dict[str, str]]:
+    candidates: List[Dict[str, str]] = []
+    images_section = message.get("images")
+    if images_section:
+        candidates.extend(_extract_images_from_content(images_section))
+    candidates.extend(_extract_images_from_content(message.get("content")))
+    candidates.extend(_extract_images_from_content(message.get("attachments")))
+    candidates.extend(_extract_images_from_content(message.get("data")))
+
+    deduped: List[Dict[str, str]] = []
+    seen: Set[Tuple[str, str]] = set()
+    for item in candidates:
+        data_url = item.get("data_url")
+        media_type = item.get("media_type") or "image/png"
+        if not isinstance(data_url, str):
+            continue
+        normalized = data_url.strip()
+        if not normalized:
+            continue
+        key = (normalized, media_type)
+        if key in seen:
+            continue
+        seen.add(key)
+        deduped.append({"data_url": normalized, "media_type": media_type})
+    return deduped
+
+
+def _diagnose_no_image_response(
+    response: Dict[str, object]
+) -> Tuple[str, Optional[str], List[str]]:
+    if not isinstance(response, dict):  # Defensive guard
+        return "empty_response", None, []
+
+    text_snippets: List[str] = []
+    finish_reasons: List[str] = []
+    for choice in response.get("choices", []) or []:
+        finish = choice.get("finish_reason") or choice.get("native_finish_reason")
+        if isinstance(finish, str) and finish:
+            finish_reasons.append(finish)
+        message = choice.get("message") or {}
+        text_snippets.extend(_extract_text_from_message(message))
+
+    combined_text = " ".join(snippet for snippet in text_snippets if snippet)
+    normalized_text = combined_text.lower().strip()
+
+    reason = "empty_response"
+    detail: Optional[str] = None
+    if normalized_text:
+        refusal_keywords = (
+            "unable",
+            "cannot",
+            "can't",
+            "not able",
+            "policy",
+            "safety",
+            "refuse",
+            "decline",
+        )
+        reason = (
+            "content_policy"
+            if any(keyword in normalized_text for keyword in refusal_keywords)
+            else "provider_text_response"
+        )
+        snippet = _truncate(combined_text, 400)
+        detail = f"OpenRouter returned text instead of images: {snippet}"
+        if finish_reasons:
+            detail += f" (finish_reason={','.join(finish_reasons)})"
+    return reason, detail, text_snippets
+
+
+def _extract_images_from_content(
+    content: object, fallback_mime: str = "image/png"
+) -> List[Dict[str, str]]:
+    if isinstance(content, list):
+        results: List[Dict[str, str]] = []
+        for entry in content:
+            results.extend(_extract_images_from_entry(entry, fallback_mime))
+        return results
+    if isinstance(content, dict):
+        return _extract_images_from_entry(content, fallback_mime)
+    return []
+
+
+def _extract_images_from_entry(
+    entry: object, fallback_mime: str = "image/png"
+) -> List[Dict[str, str]]:
+    results: List[Dict[str, str]] = []
+    if not isinstance(entry, dict):
+        return results
+
+    media_type = entry.get("mime_type") or entry.get("media_type") or fallback_mime
+
+    base64_candidates: List[Tuple[str, Optional[str]]] = []
+    for key in ("image_base64", "b64_json", "base64"):
+        value = entry.get(key)
+        if isinstance(value, str) and value.strip():
+            base64_candidates.append((value.strip(), media_type))
+
+    for key in ("dataUrl", "data_url"):
+        url_value = entry.get(key)
+        if isinstance(url_value, str) and url_value.strip():
+            results.append({"data_url": url_value.strip(), "media_type": media_type})
+
+    image_url = entry.get("image_url") or entry.get("url")
+    if isinstance(image_url, dict):
+        url = image_url.get("url")
+        if isinstance(url, str) and url.strip():
+            results.append(
+                {
+                    "data_url": url.strip(),
+                    "media_type": image_url.get("mime_type") or media_type,
+                }
+            )
+    elif isinstance(image_url, str) and image_url.strip():
+        results.append({"data_url": image_url.strip(), "media_type": media_type})
+
+    urls_field = entry.get("urls")
+    if isinstance(urls_field, list):
+        for item in urls_field:
+            if isinstance(item, str) and item.strip():
+                results.append({"data_url": item.strip(), "media_type": media_type})
+            elif isinstance(item, dict):
+                maybe_url = item.get("url")
+                if isinstance(maybe_url, str) and maybe_url.strip():
+                    results.append(
+                        {
+                            "data_url": maybe_url.strip(),
+                            "media_type": item.get("mime_type") or media_type,
+                        }
+                    )
+
+    inline_data = entry.get("inline_data")
+    if isinstance(inline_data, dict):
+        inline_mime = inline_data.get("mime_type") or media_type
+        payload = inline_data.get("data")
+        if isinstance(payload, str) and payload.strip():
+            base64_candidates.append((payload.strip(), inline_mime))
+        base64_candidates.extend(
+            _collect_base64_candidates_from_dict(inline_data, inline_mime)
+        )
+
+    data_field = entry.get("data")
+    if isinstance(data_field, dict):
+        data_mime = data_field.get("mime_type") or data_field.get("media_type") or media_type
+        base64_candidates.extend(
+            _collect_base64_candidates_from_dict(data_field, data_mime)
+        )
+        for key in ("dataUrl", "data_url"):
+            val = data_field.get(key)
+            if isinstance(val, str) and val.strip():
+                results.append({"data_url": val.strip(), "media_type": data_mime})
+        results.extend(_extract_images_from_content(data_field.get("content"), data_mime))
+
+    for payload, mime in base64_candidates:
+        results.append({"data_url": _ensure_data_url(payload, mime), "media_type": mime or fallback_mime})
+
+    for nested_key in ("content", "attachments", "output", "outputs", "results", "parts"):
+        nested = entry.get(nested_key)
+        if nested:
+            results.extend(_extract_images_from_content(nested, media_type))
+
+    return results
+
+
+def _collect_base64_candidates_from_dict(
+    payload: Dict[str, object], default_mime: Optional[str]
+) -> List[Tuple[str, Optional[str]]]:
+    results: List[Tuple[str, Optional[str]]] = []
+    for key in ("image_base64", "b64_json", "base64", "data"):
+        value = payload.get(key)
+        if isinstance(value, str) and value.strip():
+            results.append((value.strip(), payload.get("mime_type") or payload.get("media_type") or default_mime))
+    return results
+
+
+def _extract_text_from_message(message: Dict[str, object]) -> List[str]:
+    if not isinstance(message, dict):
+        return []
+    results: List[str] = []
+    content = message.get("content")
+    results.extend(_extract_text_from_content(content))
+    text_field = message.get("text")
+    if isinstance(text_field, str) and text_field.strip():
+        results.append(text_field.strip())
+    data_field = message.get("data")
+    if data_field:
+        results.extend(_extract_text_from_content(data_field))
+    return [segment for segment in results if segment]
+
+
+def _extract_text_from_content(content: object) -> List[str]:
+    if isinstance(content, str):
+        return [content.strip()]
+    if isinstance(content, list):
+        results: List[str] = []
+        for entry in content:
+            results.extend(_extract_text_from_entry(entry))
+        return results
+    if isinstance(content, dict):
+        return _extract_text_from_entry(content)
+    return []
+
+
+def _extract_text_from_entry(entry: object) -> List[str]:
+    if isinstance(entry, str):
+        stripped = entry.strip()
+        return [stripped] if stripped else []
+    if not isinstance(entry, dict):
+        return []
+
+    results: List[str] = []
+    text_field = entry.get("text")
+    if isinstance(text_field, str) and text_field.strip():
+        results.append(text_field.strip())
+
+    for key in ("title", "summary", "content"):
+        value = entry.get(key)
+        if isinstance(value, str) and value.strip():
+            results.append(value.strip())
+        elif value:
+            results.extend(_extract_text_from_content(value))
+
+    for nested_key in ("output", "outputs", "parts", "messages", "segments"):
+        nested = entry.get(nested_key)
+        if nested:
+            results.extend(_extract_text_from_content(nested))
+
+    return results
+
 
 def build_generator(output_dir: Path, kind: Optional[str] = None) -> ImageGenerator:
     normalized = (kind or "openrouter").lower()
@@ -688,7 +933,7 @@ def _shift_color(color: tuple[int, int, int], delta: int) -> tuple[int, int, int
     return tuple(max(0, min(255, channel + delta)) for channel in color)
 
 
-def _truncate(text: str, limit: int) -> str:
+def _truncate_overlay(text: str, limit: int) -> str:
     if len(text) <= limit:
         return text
     return text[: limit - 3] + "..."
